@@ -1,8 +1,8 @@
 /* root_dse.c - Provides the Root DSA-Specific Entry */
-/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.95.2.9 2005/08/12 19:25:37 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/root_dse.c,v 1.113.2.9 2009/01/22 00:01:03 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2005 The OpenLDAP Foundation.
+ * Copyright 1999-2009 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,21 +28,153 @@
 #include "slapi/slapi.h"
 #endif
 
-static struct berval supportedFeatures[] = {
+static struct berval	builtin_supportedFeatures[] = {
 	BER_BVC(LDAP_FEATURE_MODIFY_INCREMENT),		/* Modify/increment */
-	BER_BVC(LDAP_FEATURE_ALL_OP_ATTRS),			/* All Op Attrs (+) */
+	BER_BVC(LDAP_FEATURE_ALL_OP_ATTRS),		/* All Op Attrs (+) */
 	BER_BVC(LDAP_FEATURE_OBJECTCLASS_ATTRS),	/* OCs in Attrs List (@class) */
 	BER_BVC(LDAP_FEATURE_ABSOLUTE_FILTERS),		/* (&) and (|) search filters */
-	BER_BVC(LDAP_FEATURE_LANGUAGE_TAG_OPTIONS), /* Language Tag Options */
-	BER_BVC(LDAP_FEATURE_LANGUAGE_RANGE_OPTIONS),/* Language Range Options */
-#ifdef LDAP_FEATURE_SUBORDINATE_SCOPE
+	BER_BVC(LDAP_FEATURE_LANGUAGE_TAG_OPTIONS),	/* Language Tag Options */
+	BER_BVC(LDAP_FEATURE_LANGUAGE_RANGE_OPTIONS),	/* Language Range Options */
+#ifdef LDAP_DEVEL
 	BER_BVC(LDAP_FEATURE_SUBORDINATE_SCOPE),	/* "children" search scope */
 #endif
-	{0,NULL}
+	BER_BVNULL
 };
+static struct berval	*supportedFeatures;
 
 static Entry	*usr_attr = NULL;
 
+/*
+ * allow modules to register functions that muck with the root DSE entry
+ */
+
+typedef struct entry_info_t {
+	SLAP_ENTRY_INFO_FN	func;
+	void			*arg;
+	struct entry_info_t	*next;
+} entry_info_t;
+
+static entry_info_t *extra_info;
+
+int
+entry_info_register( SLAP_ENTRY_INFO_FN func, void *arg )
+{
+	entry_info_t	*ei = ch_calloc( 1, sizeof( entry_info_t ) );
+
+	ei->func = func;
+	ei->arg = arg;
+
+	ei->next = extra_info;
+	extra_info = ei;
+
+	return 0;
+}
+
+int
+entry_info_unregister( SLAP_ENTRY_INFO_FN func, void *arg )
+{
+	entry_info_t	**eip;
+
+	for ( eip = &extra_info; *eip != NULL; eip = &(*eip)->next ) {
+		if ( (*eip)->func == func && (*eip)->arg == arg ) {
+			entry_info_t	*ei = *eip;
+
+			*eip = ei->next;
+
+			ch_free( ei );
+
+			return 0;
+		}
+	}
+
+	return -1;
+}
+
+void
+entry_info_destroy( void )
+{
+	entry_info_t	**eip;
+
+	for ( eip = &extra_info; *eip != NULL;  ) {
+		entry_info_t	*ei = *eip;
+
+		eip = &(*eip)->next;
+
+		ch_free( ei );
+	}
+}
+
+/*
+ * Allow modules to register supported features
+ */
+
+static int
+supported_feature_init( void )
+{
+	int		i;
+
+	if ( supportedFeatures != NULL ) {
+		return 0;
+	}
+
+	for ( i = 0; !BER_BVISNULL( &builtin_supportedFeatures[ i ] ); i++ )
+		;
+
+	supportedFeatures = ch_calloc( sizeof( struct berval ), i + 1 );
+	if ( supportedFeatures == NULL ) {
+		return -1;
+	}
+
+	for ( i = 0; !BER_BVISNULL( &builtin_supportedFeatures[ i ] ); i++ ) {
+		ber_dupbv( &supportedFeatures[ i ], &builtin_supportedFeatures[ i ] );
+	}
+	BER_BVZERO( &supportedFeatures[ i ] );
+
+	return 0;
+}
+
+int
+supported_feature_destroy( void )
+{
+	int		i;
+
+	if ( supportedFeatures == NULL ) {
+		return 0;
+	}
+	
+	for ( i = 0; !BER_BVISNULL( &supportedFeatures[ i ] ); i++ ) {
+		ch_free( supportedFeatures[ i ].bv_val );
+	}
+
+	ch_free( supportedFeatures );
+	supportedFeatures = NULL;
+
+	return 0;
+}
+
+int
+supported_feature_load( struct berval *f )
+{
+	struct berval	*tmp;
+	int		i;
+
+	supported_feature_init();
+
+	for ( i = 0; !BER_BVISNULL( &supportedFeatures[ i ] ); i++ )
+		;
+
+	tmp = ch_realloc( supportedFeatures, sizeof( struct berval ) * ( i + 2 ) );
+	if ( tmp == NULL ) {
+		return -1;
+	}
+	supportedFeatures = tmp;
+
+	ber_dupbv( &supportedFeatures[ i ], f );
+	BER_BVZERO( &supportedFeatures[ i + 1 ] );
+
+	return 0;
+}
+
 int
 root_dse_info(
 	Connection *conn,
@@ -81,11 +213,10 @@ root_dse_info(
 	AttributeDescription *ad_ref
 		= slap_schema.si_ad_ref;
 
-	e = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
-
+	e = entry_alloc();
 	if( e == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"root_dse_info: SLAP_CALLOC failed", 0, 0, 0 );
+			"root_dse_info: entry_alloc failed", 0, 0, 0 );
 		return LDAP_OTHER;
 	}
 
@@ -101,17 +232,20 @@ root_dse_info(
 
 	e->e_private = NULL;
 
+	/* FIXME: is this really needed? */
 	BER_BVSTR( &val, "top" );
 	if( attr_merge_one( e, ad_objectClass, &val, NULL ) ) {
+fail:
+		entry_free( e );
 		return LDAP_OTHER;
 	}
 
 	BER_BVSTR( &val, "OpenLDAProotDSE" );
 	if( attr_merge_one( e, ad_objectClass, &val, NULL ) ) {
-		return LDAP_OTHER;
+		goto fail;
 	}
 	if( attr_merge_one( e, ad_structuralObjectClass, &val, NULL ) ) {
-		return LDAP_OTHER;
+		goto fail;
 	}
 
 	LDAP_STAILQ_FOREACH( be, &backendDB, be_next ) {
@@ -125,7 +259,7 @@ root_dse_info(
 					&be->be_suffix[0],
 					&be->be_nsuffix[0] ) )
 			{
-				return LDAP_OTHER;
+				goto fail;
 			}
 			continue;
 		}
@@ -134,7 +268,7 @@ root_dse_info(
 					&be->be_suffix[0],
 					& be->be_nsuffix[0] ) )
 			{
-				return LDAP_OTHER;
+				goto fail;
 			}
 			continue;
 		}
@@ -146,7 +280,7 @@ root_dse_info(
 					&be->be_suffix[j],
 					&be->be_nsuffix[0] ) )
 			{
-				return LDAP_OTHER;
+				goto fail;
 			}
 		}
 	}
@@ -155,26 +289,30 @@ root_dse_info(
 
 	/* supportedControl */
 	if ( controls_root_dse_info( e ) != 0 ) {
-		return LDAP_OTHER;
+		goto fail;
 	}
 
 	/* supportedExtension */
 	if ( exop_root_dse_info( e ) != 0 ) {
-		return LDAP_OTHER;
+		goto fail;
 	}
 
 #ifdef LDAP_SLAPI
 	/* netscape supportedExtension */
 	for ( i = 0; (bv = slapi_int_get_supported_extop(i)) != NULL; i++ ) {
 		if( attr_merge_one( e, ad_supportedExtension, bv, NULL ) ) {
-			return LDAP_OTHER;
+			goto fail;
 		}
 	}
 #endif /* LDAP_SLAPI */
 
 	/* supportedFeatures */
+	if ( supportedFeatures == NULL ) {
+		supported_feature_init();
+	}
+
 	if( attr_merge( e, ad_supportedFeatures, supportedFeatures, NULL ) ) {
-		return LDAP_OTHER;
+		goto fail;
 	}
 
 	/* supportedLDAPVersion */
@@ -184,12 +322,12 @@ root_dse_info(
 		 * clients.
 		 */
 	for ( i=LDAP_VERSION3; i<=LDAP_VERSION_MAX; i++ ) {
-		char buf[BUFSIZ];
+		char buf[sizeof("255")];
 		snprintf(buf, sizeof buf, "%d", i);
 		val.bv_val = buf;
 		val.bv_len = strlen( val.bv_val );
 		if( attr_merge_one( e, ad_supportedLDAPVersion, &val, NULL ) ) {
-			return LDAP_OTHER;
+			goto fail;
 		}
 	}
 
@@ -201,7 +339,8 @@ root_dse_info(
 			val.bv_val = supportedSASLMechanisms[i];
 			val.bv_len = strlen( val.bv_val );
 			if( attr_merge_one( e, ad_supportedSASLMechanisms, &val, NULL ) ) {
-				return LDAP_OTHER;
+				ldap_charray_free( supportedSASLMechanisms );
+				goto fail;
 			}
 		}
 		ldap_charray_free( supportedSASLMechanisms );
@@ -209,7 +348,7 @@ root_dse_info(
 
 	if ( default_referral != NULL ) {
 		if( attr_merge( e, ad_ref, default_referral, NULL /* FIXME */ ) ) {
-			return LDAP_OTHER;
+			goto fail;
 		}
 	}
 
@@ -219,22 +358,48 @@ root_dse_info(
 			if( attr_merge( e, a->a_desc, a->a_vals,
 				(a->a_nvals == a->a_vals) ? NULL : a->a_nvals ) )
 			{
-				return LDAP_OTHER;
+				goto fail;
 			}
 		}
 	}
 
+	if ( extra_info ) {
+		entry_info_t	*ei = extra_info;
+
+		for ( ; ei; ei = ei->next ) {
+			ei->func( ei->arg, e );
+		}
+	}
+
 	*entry = e;
 	return LDAP_SUCCESS;
 }
 
+int
+root_dse_init( void )
+{
+	return 0;
+}
+
+int
+root_dse_destroy( void )
+{
+	if ( usr_attr ) {
+		entry_free( usr_attr );
+		usr_attr = NULL;
+	}
+
+	return 0;
+}
+
 /*
  * Read the entries specified in fname and merge the attributes
  * to the user defined rootDSE. Note thaat if we find any errors
  * what so ever, we will discard the entire entries, print an
  * error message and return.
  */
-int read_root_dse_file( const char *fname )
+int
+root_dse_read_file( const char *fname )
 {
 	struct LDIFFP	*fp;
 	int rc = 0, lineno = 0, lmax = 0;
@@ -242,16 +407,16 @@ int read_root_dse_file( const char *fname )
 
 	if ( (fp = ldif_open( fname, "r" )) == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"could not open rootdse attr file \"%s\" - absolute path?\n",
+			"root_dse_read_file: could not open rootdse attr file \"%s\" - absolute path?\n",
 			fname, 0, 0 );
 		perror( fname );
 		return EXIT_FAILURE;
 	}
 
-	usr_attr = (Entry *) SLAP_CALLOC( 1, sizeof(Entry) );
+	usr_attr = entry_alloc();
 	if( usr_attr == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
-			"read_root_dse_file: SLAP_CALLOC failed", 0, 0, 0 );
+			"root_dse_read_file: entry_alloc failed", 0, 0, 0 );
 		ldif_close( fp );
 		return LDAP_OTHER;
 	}
@@ -262,17 +427,19 @@ int read_root_dse_file( const char *fname )
 		Attribute *a;
 
 		if( e == NULL ) {
-			fprintf( stderr, "root_dse: could not parse entry (line=%d)\n",
-				lineno );
+			Debug( LDAP_DEBUG_ANY, "root_dse_read_file: "
+				"could not parse entry (file=\"%s\" line=%d)\n",
+				fname, lineno, 0 );
 			rc = EXIT_FAILURE;
 			break;
 		}
 
 		/* make sure the DN is the empty DN */
 		if( e->e_nname.bv_len ) {
-			fprintf( stderr,
-				"root_dse: invalid rootDSE - dn=\"%s\" (line=%d)\n",
-				e->e_dn, lineno );
+			Debug( LDAP_DEBUG_ANY,
+				"root_dse_read_file: invalid rootDSE "
+				"- dn=\"%s\" (file=\"%s\" line=%d)\n",
+				e->e_dn, fname, lineno );
 			entry_free( e );
 			rc = EXIT_FAILURE;
 			break;
@@ -306,44 +473,29 @@ int read_root_dse_file( const char *fname )
 
 	ldif_close( fp );
 
-	Debug(LDAP_DEBUG_CONFIG, "rootDSE file %s read.\n", fname, 0, 0);
+	Debug(LDAP_DEBUG_CONFIG, "rootDSE file=\"%s\" read.\n", fname, 0, 0);
 	return rc;
 }
 
 int
 slap_discover_feature(
-	const char	*uri,
-	int		version,
+	slap_bindconf	*sb,
 	const char	*attr,
 	const char	*val )
 {
-	LDAP		*ld;
+	LDAP		*ld = NULL;
 	LDAPMessage	*res = NULL, *entry;
 	int		rc, i;
-	struct berval	cred = BER_BVC( "" ),
-			bv_val,
+	struct berval	bv_val,
 			**values = NULL;
 	char		*attrs[ 2 ] = { NULL, NULL };
 
-	ber_str2bv( val, 0, 0, &bv_val );
+	rc = slap_client_connect( &ld, sb );
+	if ( rc != LDAP_SUCCESS ) {
+		goto done;
+	}
+
 	attrs[ 0 ] = (char *) attr;
-
-	rc = ldap_initialize( &ld, uri );
-	if ( rc != LDAP_SUCCESS ) {
-		return rc;
-	}
-
-	rc = ldap_set_option( ld, LDAP_OPT_PROTOCOL_VERSION, &version );
-	if ( rc != LDAP_SUCCESS ) {
-		goto done;
-	}
-
-	rc = ldap_sasl_bind_s( ld, "", LDAP_SASL_SIMPLE,
-			&cred, NULL, NULL, NULL );
-	if ( rc != LDAP_SUCCESS ) {
-		goto done;
-	}
-
 	rc = ldap_search_ext_s( ld, "", LDAP_SCOPE_BASE, "(objectClass=*)",
 			attrs, 0, NULL, NULL, NULL, 0, &res );
 	if ( rc != LDAP_SUCCESS ) {
@@ -361,6 +513,7 @@ slap_discover_feature(
 		goto done;
 	}
 
+	ber_str2bv( val, 0, 0, &bv_val );
 	for ( i = 0; values[ i ] != NULL; i++ ) {
 		if ( bvmatch( &bv_val, values[ i ] ) ) {
 			rc = LDAP_COMPARE_TRUE;
